import zmq
import argparse

def _get_socket(typ):
    """
    Create a ZeroMQ socket of type typ.
    """
    context = zmq.Context()
    socket = context.socket(typ)
    return socket


def _get_bound_socket(typ, uri):
    """
    Create a ZeroMQ socket of type typ and bind it to uri.
    """
    socket = _get_socket(typ)
    socket.bind(uri)
    return socket


def _get_connected_socket(typ, uri):
    """
    Create a ZeroMQ socket of type typ and connect it to uri.
    """
    socket = _get_socket(typ)
    socket.connect(uri)
    return socket


def pub(uri):
    """
    Publish stdin on a ZeroMQ socket bound to uri.
    """
    socket = _get_bound_socket(zmq.PUB, uri)
    while True:
        socket.send_unicode(unicode("%s%s" % (key, raw_input())))


def sub(uri):
    """
    Subscribe to a ZeroMQ PUB socket at uri and print its messages to stdout.
    """
    socket = _get_connected_socket(zmq.SUB, uri)
    socket.setsockopt_string(zmq.SUBSCRIBE, unicode(key))
    while True:
        print socket.recv()


def push(uri):
    """
    Push stdin to a ZeroMQ PULL socket at uri.
    """
    socket = _get_connected_socket(zmq.PUSH, uri)
    while True:
        socket.send_unicode(unicode(raw_input()))


def pull(uri):
    """
    Create a ZeroMQ PULL socket at uri and print its messages to stdout.
    """
    socket = _get_bound_socket(zmq.PULL, uri)
    while True:
        print socket.recv()


def main():
    types = {
        "pub": pub,
        "sub": sub,
        "push": push,
        "pull": pull
    }

    p = argparse.ArgumentParser()
    p.add_argument("type", choices=types.keys())
    p.add_argument("uri")
    p.add_argument("--key", default="ZMCAT")
    args = p.parse_args()

    # Set key globally
    key = args.key

    try:
        # Call the relevant function
        types[args.type](args.uri)
    except (EOFError, KeyboardInterrupt):
        pass


if __name__ == "__main__":
    main()


import os
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="zmcat",
    version="0.0.4",
    author="Ellis Percival",
    author_email="zmcat@failcode.co.uk",
    description="A simple command line tool to test ZMQ push/pull/pub/sub soc"
        "kets. Based on https://github.com/lucasdicioccio/zmcat",
    license="UNLICENSE",
    keywords="zmq zeromq netcat nc 0mq",
    url="https://github.com/flyte/zmcat",
    packages=("zmcat",),
    install_requires=read("requirements.txt"),
    entry_points={
        "console_scripts": (
            "zmcat = zmcat.tool:main",
        )
    }
)
